"""Utility methods for the AI chat"""

from ol_openedx_chat.constants import CHAT_APPLICABLE_BLOCKS


def is_aside_applicable_to_block(block):
    """Check if the xBlock should support AI Chat"""
    return getattr(block, "category", None) in CHAT_APPLICABLE_BLOCKS



"""
Compatibility layer to isolate core-platform method calls from implementation.
"""

WAFFLE_FLAG_NAMESPACE = "ol_openedx_chat"

# .. toggle_name: ol_openedx_chat.ol_openedx_chat_enabled
# .. toggle_implementation: CourseWaffleFlag
# .. toggle_default: False
# .. toggle_description: Enables ol_openedx_chat plugin for a course.
# .. toggle_use_cases: open_edx
# .. toggle_creation_date: 2025-02-26
# .. toggle_tickets: None
# .. toggle_warning: None.
OL_OPENEDX_CHAT_ENABLED = "ol_openedx_chat_enabled"


def get_ol_openedx_chat_enabled_flag():
    """
    Import and return Waffle flag for enabling ol_openedx_chat.
    """
    from openedx.core.djangoapps.waffle_utils import CourseWaffleFlag

    return CourseWaffleFlag(
        f"{WAFFLE_FLAG_NAMESPACE}.{OL_OPENEDX_CHAT_ENABLED}", __name__
    )



# noqa: INP001

"""Settings to provide to edX"""


def plugin_settings(settings):
    """
    Populate common settings
    """
    env_tokens = getattr(settings, "ENV_TOKENS", {})
    settings.LEARN_AI_API_URL = env_tokens.get("LEARN_AI_API_URL", "")



"""
ol_openedx_chat Django application initialization.
"""

from django.apps import AppConfig
from edx_django_utils.plugins import PluginSettings
from openedx.core.djangoapps.plugins.constants import ProjectType, SettingsType


class OLOpenedxChatConfig(AppConfig):
    """
    Configuration for the ol_openedx_chat Django application.
    """

    name = "ol_openedx_chat"

    plugin_app = {
        PluginSettings.CONFIG: {
            ProjectType.LMS: {
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "settings.common"},
                SettingsType.DEVSTACK: {
                    PluginSettings.RELATIVE_PATH: "settings.devstack"
                },
            },
            ProjectType.CMS: {
                SettingsType.COMMON: {PluginSettings.RELATIVE_PATH: "settings.common"},
                SettingsType.DEVSTACK: {
                    PluginSettings.RELATIVE_PATH: "settings.devstack"
                },
            },
        },
    }



# The dictionary should contain all the block types for which the chat should be
# applicable if a block has sub-blocks or sub category, that should be added in the list
VIDEO_BLOCK_CATEGORY = "video"
PROBLEM_BLOCK_CATEGORY = "problem"

# The actual chat URL is `https://api-learn-ai.ol.mit.edu/http/video_gpt_agent/`
# for video blocks and`https://api-learn-ai.ol.mit.edu/http/tutor_agent/`
# for problem blocks.
MIT_AI_CHAT_URL_PATHS = {
    VIDEO_BLOCK_CATEGORY: "http/video_gpt_agent/",
    PROBLEM_BLOCK_CATEGORY: "http/tutor_agent/",
}
CHAT_APPLICABLE_BLOCKS = [PROBLEM_BLOCK_CATEGORY, VIDEO_BLOCK_CATEGORY]

ENGLISH_LANGUAGE_TRANSCRIPT = "en"



import logging

import pkg_resources
from django.conf import settings
from django.template import Context, Template
from django.utils.translation import gettext_lazy as _
from ol_openedx_chat.compat import get_ol_openedx_chat_enabled_flag
from ol_openedx_chat.constants import (
    ENGLISH_LANGUAGE_TRANSCRIPT,
    MIT_AI_CHAT_URL_PATHS,
    VIDEO_BLOCK_CATEGORY,
)
from ol_openedx_chat.utils import is_aside_applicable_to_block
from rest_framework import status as api_status
from web_fragments.fragment import Fragment
from webob.response import Response
from xblock.core import XBlock, XBlockAside
from xblock.fields import Boolean, Scope
from xmodule.x_module import AUTHOR_VIEW, STUDENT_VIEW

log = logging.getLogger(__name__)


def get_resource_bytes(path):
    """
    Helper method to get the unicode contents of a resource in this repo.

    Args:
        path (str): The path of the resource

    Returns:
        unicode: The unicode contents of the resource at the given path
    """  # noqa: D401
    resource_contents = pkg_resources.resource_string(__name__, path)
    return resource_contents.decode("utf-8")


def render_template(template_path, context=None):
    """
    Evaluate a template by resource path, applying the provided context.
    """
    context = context or {}
    template_str = get_resource_bytes(template_path)
    template = Template(template_str)
    return template.render(Context(context))


class OLChatAside(XBlockAside):
    """
    XBlock aside that enables OL AI Chat functionality for an XBlock
    """

    ol_chat_enabled = Boolean(
        display_name=_("Open Learning Chat enabled status"),
        default=False,
        scope=Scope.settings,
        help=_("Indicates whether or not Open Learning chat is enabled for a block"),
    )

    @XBlockAside.aside_for(STUDENT_VIEW)
    def student_view_aside(self, block, context=None):
        """
        Renders the aside contents for the student view
        """  # noqa: D401
        from xmodule.video_block.transcripts_utils import Transcript

        # This is a workaround for those blocks which do not have has_author_view=True
        # because when a block does not define has_author_view=True in it, the only view
        # that gets rendered is student_view in place of author view.

        if getattr(self.runtime, "is_author_mode", False):
            return self.author_view_aside(block, context)

        fragment = Fragment("")
        if not self.ol_chat_enabled:
            return fragment

        block_usage_key = self.scope_ids.usage_id.usage_key
        block_id = block_usage_key.block_id
        block_type = getattr(block, "category", None)

        fragment.add_content(
            render_template(
                "static/html/student_view.html",
                {
                    "block_id": block_id,
                    "block_usage_key": block_usage_key,
                    "block_type": block_type,
                },
            )
        )
        fragment.add_css(get_resource_bytes("static/css/ai_chat.css"))
        fragment.add_javascript(get_resource_bytes("static/js/ai_chat.js"))

        request_body = {
            "edx_module_id": block_usage_key,
        }

        if block_type == VIDEO_BLOCK_CATEGORY:
            try:
                transcripts_info = block.get_transcripts_info()
                if transcripts_info.get("transcripts", {}).get(
                    ENGLISH_LANGUAGE_TRANSCRIPT
                ):
                    request_body["transcript_asset_id"] = Transcript.asset_location(
                        block.location,
                        transcripts_info["transcripts"][ENGLISH_LANGUAGE_TRANSCRIPT],
                    )

            except Exception:  # noqa: BLE001
                log.info(
                    "Error while fetching transcripts for block %s",
                    block.location,
                )

        extra_context = {
            "ask_tim_drawer_title": f"about {block.display_name}",
            "user_id": self.runtime.user_id,
            "block_id": block_id,
            "block_type": block_type,
            "edx_module_id": block_usage_key,
            "chat_api_url": f"{settings.LEARN_AI_API_URL}/{MIT_AI_CHAT_URL_PATHS[block_type]}",  # noqa: E501
            "learning_mfe_base_url": settings.LEARNING_MICROFRONTEND_URL,
            "request_body": request_body,
        }

        fragment.initialize_js("AiChatAsideInit", json_args=extra_context)
        return fragment

    @XBlockAside.aside_for(AUTHOR_VIEW)
    def author_view_aside(self, block, context=None):  # noqa: ARG002
        """
        Renders the aside contents for the author view
        """  # noqa: D401
        fragment = Fragment("")
        fragment.add_content(
            render_template(
                "static/html/studio_view.html",
                {
                    "is_enabled": self.ol_chat_enabled,
                    "block_id": block.location.block_id,  # Passing this along as a unique key for checkboxes  # noqa: E501
                },
            )
        )
        fragment.add_css(get_resource_bytes("static/css/studio.css"))
        fragment.add_javascript(get_resource_bytes("static/js/studio.js"))
        fragment.initialize_js("OLChatInit")
        return fragment

    @classmethod
    def should_apply_to_block(cls, block):
        """
        Overrides base XBlockAside implementation. Indicates whether or not this aside
        should apply to a given block.

        Due to the different ways that the Studio and LMS runtimes construct XBlock
        instances, the problem type of the given block needs to be retrieved in
        different ways.
        """  # noqa: D401
        return get_ol_openedx_chat_enabled_flag().is_enabled(
            block.scope_ids.usage_id.context_key
        ) and is_aside_applicable_to_block(block=block)

    @XBlock.handler
    def update_chat_config(self, request, suffix=""):  # noqa: ARG002
        """Update the chat configurations"""
        try:
            posted_data = request.json
        except ValueError:
            return Response(
                "Invalid request body", status=api_status.HTTP_400_BAD_REQUEST
            )

        self.ol_chat_enabled = posted_data.get("is_enabled", False)
        return Response()



"""
URLs for ol_openedx_chat.
"""

urlpatterns = [  # type: ignore[var-annotated]
    # Fill in URL patterns and views here.
    # re_path(r'', TemplateView.as_view(template_name="ol_openedx_chat/base.html")),  # noqa: ERA001, E501
]




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/ol_openedx_chat:ol_openedx_chat_package

from setuptools import setup

setup(**{
    'author': 'MIT Office of Digital Learning',
    'description': 'An Open edX plugin to add Open Learning AI chat aside to xBlocks',
    'entry_points': {
        'xblock_asides.v1': [
            'ol_openedx_chat = ol_openedx_chat.block:OLChatAside',
        ],
        'lms.djangoapp': [
            'ol_openedx_chat = ol_openedx_chat.apps:OLOpenedxChatConfig',
        ],
        'cms.djangoapp': [
            'ol_openedx_chat = ol_openedx_chat.apps:OLOpenedxChatConfig',
        ],
    },
    'include_package_data': True,
    'install_requires': (
        'Django>2.0',
        'XBlock',
        'djangorestframework<4.0.0,>=3.14.0',
        'edx-django-utils>4.0.0',
    ),
    'keywords': 'Python edx',
    'license': 'BSD-3-Clause',
    'name': 'ol-openedx-chat',
    'namespace_packages': (
    ),
    'package_data': {
        'ol_openedx_chat': (
            'static/css/ai_chat.css',
            'static/css/studio.css',
            'static/html/student_view.html',
            'static/html/studio_view.html',
            'static/images/icon.svg',
            'static/js/ai_chat.js',
            'static/js/studio.js',
        ),
    },
    'packages': (
        '',
        'ol_openedx_chat',
        'ol_openedx_chat.settings',
    ),
    'python_requires': '>=3.8',
    'version': '0.2.7',
    'zip_safe': False,
})
